package cn.estsh.i3plus.ext.mes.pcn.apiservice.daoimpl;


import cn.estsh.i3plus.ext.mes.pcn.apiservice.dao.IMesEquipmentLogDao;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesEquipVariableCollectContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentVariable;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class MesEquipmentLogDao implements IMesEquipmentLogDao {

    @Value("${mes.sharding.append.org:}")
    public String mesShardingAppendOrg;

    @Autowired
    private EntityManager entityManager;

    @Override
    public List<MesEquipVariableCollectContext> queryMesEquipmentLog(List<MesEquipmentVariable> equipmentVariableList, String organizeCode, Integer equipId, Integer variableType) {

        List<MesEquipVariableCollectContext> equipVariableCollectContextList = generateEquipVariableCollectContextList(equipmentVariableList);

        if (CollectionUtils.isEmpty(equipmentVariableList) || StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipId) || StringUtils.isEmpty(variableType)) return null;

        StringBuffer builder = new StringBuffer();

        builder.append(" select e.equip_variable_id, e.equip_variable_value, e.equip_variable_status, e.quality, e.modify_date_time, e.id ");
        builder.append(" from mes_equipment_log_");
        if (!StringUtils.isEmpty(mesShardingAppendOrg) && mesShardingAppendOrg.toUpperCase().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.name())) builder.append(organizeCode.toLowerCase()).append(MesPcnExtConstWords.E_UNDERLINE);
        builder.append(equipId);
        builder.append(" e where e.organize_code = :organizeCode and e.variable_type = :variableType ");

        Query query = entityManager.createNativeQuery(builder.toString());
        query.setParameter(MesPcnExtConstWords.ORGANIZE_CODE, organizeCode);
        query.setParameter(MesPcnExtConstWords.VARIABLE_TYPE, variableType);
        List list = query.getResultList();

        if (CollectionUtils.isEmpty(list)) return equipVariableCollectContextList;

        List<String> equipVariableIdList = equipmentVariableList.stream().filter(o -> null != o).map(o -> o.getId().toString()).collect(Collectors.toList());

        for (Object equipmentLog : list) {

            if (null == equipmentLog) continue;

            Object[] equipmentLogArr = (Object[]) equipmentLog;

            if (null == equipmentLogArr || equipmentLogArr.length != 6 || StringUtils.isEmpty(equipmentLogArr[0]) || !equipVariableIdList.contains(equipmentLogArr[0].toString())) continue;

            equipVariableCollectContextList.stream().filter(o -> (null != o && o.getEquipVariableId().toString().equals(equipmentLogArr[0].toString()))).findFirst().get().copyValue(equipmentLogArr[1], equipmentLogArr[2], equipmentLogArr[3], equipmentLogArr[4], equipmentLogArr[5]);

        }

        return equipVariableCollectContextList;
    }

    @Override
    public void updateEquipVariableStatus(String organizeCode, Integer equipId, Map<Long, String> equipmentLogIdMap, Integer equipVariableStatus) {

        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipId) || CollectionUtils.isEmpty(equipmentLogIdMap) || StringUtils.isEmpty(equipVariableStatus)) return;

        StringBuffer builder = new StringBuffer();

        for (int i = 0; i < equipmentLogIdMap.size(); i ++) {

            builder.append(" update mes_equipment_log_");
            if (!StringUtils.isEmpty(mesShardingAppendOrg) && mesShardingAppendOrg.toUpperCase().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.name())) builder.append(organizeCode.toLowerCase()).append(MesPcnExtConstWords.E_UNDERLINE);
            builder.append(equipId);
            builder.append(" set equip_variable_status = :equipVariableStatus_");
            builder.append(i);
            builder.append(" where id = :id_");
            builder.append(i);
            builder.append(" and modify_date_time = :modifyDatetime_");
            builder.append(i);
            builder.append(" ;");

        }

        Query update = entityManager.createNativeQuery(builder.toString());

        Integer index = 0;
        for (Map.Entry<Long, String> entry : equipmentLogIdMap.entrySet()) {
            if (null == entry) continue;
            update.setParameter(String.format("%s_%s", MesPcnExtConstWords.EQUIP_VARIABLE_STATUS, index), equipVariableStatus);
            update.setParameter(String.format("%s_%s", MesPcnExtConstWords.ID, index), entry.getKey());
            update.setParameter(String.format("%s_%s", MesPcnExtConstWords.MODIFY_DATE_TIME, index), entry.getValue());
            index ++;
        }

        update.executeUpdate();

    }

    private List<MesEquipVariableCollectContext> generateEquipVariableCollectContextList(List<MesEquipmentVariable> equipmentVariableList) {
        List<MesEquipVariableCollectContext> equipVariableCollectContextList = new ArrayList<>();
        if (CollectionUtils.isEmpty(equipmentVariableList)) return equipVariableCollectContextList;
        equipmentVariableList.stream().filter(o -> null != o).forEach(o -> {
            MesEquipVariableCollectContext equipVariableCollectContext = new MesEquipVariableCollectContext(o.getId());
            BeanUtils.copyProperties(o, equipVariableCollectContext);
            equipVariableCollectContextList.add(equipVariableCollectContext);
        });
        return equipVariableCollectContextList;
    }

    @Override
    public List<MesEquipVariableCollectContext> queryMesEquipmentLog(String organizeCode, Integer equipId, Integer variableType, List<Long> equipVariableIdList) {

        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipId) || StringUtils.isEmpty(variableType) || CollectionUtils.isEmpty(equipVariableIdList)) return null;

        equipVariableIdList = equipVariableIdList.stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());

        StringBuffer builder = new StringBuffer();

        builder.append(" select e.equip_variable_id, e.equip_variable_value, e.equip_variable_status, e.quality, e.modify_date_time, e.id ");
        builder.append(" from mes_equipment_log_");
        if (!StringUtils.isEmpty(mesShardingAppendOrg) && mesShardingAppendOrg.toUpperCase().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.name())) builder.append(organizeCode.toLowerCase()).append(MesPcnExtConstWords.E_UNDERLINE);
        builder.append(equipId);
        builder.append(" e where e.organize_code = :organizeCode and e.variable_type = :variableType ");
        if (equipVariableIdList.size() == 1) builder.append(" and e.equip_variable_id = :equipVariableId ");
        else builder.append(" and e.equip_variable_id in (:equipVariableId) ");

        Query query = entityManager.createNativeQuery(builder.toString());
        query.setParameter(MesPcnExtConstWords.ORGANIZE_CODE, organizeCode);
        query.setParameter(MesPcnExtConstWords.VARIABLE_TYPE, variableType);
        if (equipVariableIdList.size() == 1) query.setParameter(MesPcnExtConstWords.EQUIP_VARIABLE_ID, equipVariableIdList.get(0));
        else query.setParameter(MesPcnExtConstWords.EQUIP_VARIABLE_ID, equipVariableIdList);
        List list = query.getResultList();

        return backEquipVariableCollectContextList(list);
    }

    @Override
    public List<MesEquipVariableCollectContext> queryMesEquipmentLog(String organizeCode, Integer equipId, Integer variableType) {

        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipId) || StringUtils.isEmpty(variableType)) return null;

        StringBuffer builder = new StringBuffer();

        builder.append(" select e.equip_variable_id, e.equip_variable_value, e.equip_variable_status, e.quality, e.modify_date_time, e.id ");
        builder.append(" from mes_equipment_log_");
        if (!StringUtils.isEmpty(mesShardingAppendOrg) && mesShardingAppendOrg.toUpperCase().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.name())) builder.append(organizeCode.toLowerCase()).append(MesPcnExtConstWords.E_UNDERLINE);
        builder.append(equipId);
        builder.append(" e where e.organize_code = :organizeCode and e.variable_type = :variableType ");

        Query query = entityManager.createNativeQuery(builder.toString());
        query.setParameter(MesPcnExtConstWords.ORGANIZE_CODE, organizeCode);
        query.setParameter(MesPcnExtConstWords.VARIABLE_TYPE, variableType);
        List list = query.getResultList();

        return backEquipVariableCollectContextList(list);

    }

    private List<MesEquipVariableCollectContext> backEquipVariableCollectContextList(List list) {

        if (CollectionUtils.isEmpty(list)) return null;

        List<MesEquipVariableCollectContext> equipVariableCollectContextList = new ArrayList<>();

        for (Object equipmentLog : list) {

            if (null == equipmentLog) continue;

            Object[] equipmentLogArr = (Object[]) equipmentLog;

            if (null == equipmentLogArr || equipmentLogArr.length != 6 || StringUtils.isEmpty(equipmentLogArr[0])) continue;

            equipVariableCollectContextList.add(new MesEquipVariableCollectContext(Long.valueOf(equipmentLogArr[0].toString())).copyValue(equipmentLogArr[1], equipmentLogArr[2], equipmentLogArr[3], equipmentLogArr[4], equipmentLogArr[5]));

        }

        return equipVariableCollectContextList;
    }

}
